package ch.othello.openscep;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;

import org.bouncycastle.asn1.x509.*;
import org.bouncycastle.asn1.util.*;


import ch.othello.openscep.internal.Log;

/** Scep Client
 *
 * <pre>
 * $Id: CmdClient.java,v 1.31 2002/02/27 04:21:27 urut Exp $
 * $Log: CmdClient.java,v $
 * Revision 1.31  2002/02/27 04:21:27  urut
 * first working spkac version
 *
 * Revision 1.30  2002/02/27 04:09:36  alani
 * loadCA now only accepts a String
 *
 * Revision 1.29  2002/02/25 04:45:35  urut
 * set subjectScep from cmd line to test out various effects
 *
 * Revision 1.28  2002/02/22 13:40:00  urut
 * v2 with spkac (sending)
 *
 * Revision 1.27  2002/02/22 12:37:48  urut
 * read new cmd line parameter certificate (URL that tells where the ca signed certificate should be written to)
 *
 * Revision 1.26  2002/02/22 10:35:31  urut
 * first v2 working release (pkcs#10 only)
 * major mods,in particular:
 * - md5 hash calculation for proxyIdentifier
 * - requestPayload, Attributes is tagged object
 *
 * Revision 1.25  2002/02/22 07:32:24  alani
 * moved controll logic for certReq to the certReq command class
 *
 * Revision 1.24  2002/02/22 03:18:06  urut
 * adjusted scep library to openscep 0.4.0:
 * -proxyIdentification Envelope Removed
 * -calculation of hash for requestPayload changed
 * -messageTyp in v2 is different
 *
 * Revision 1.23  2002/02/16 01:14:45  urut
 * replaced all read-only accessed files with URL's file:///......
 * removed option caCert , only caCertURL is available now
 *
 * Revision 1.22  2002/02/16 00:40:20  urut
 * awfull lot of fixes,among them:
 * - computation of transactionID for v2
 * - spkac processing in cmd line client
 * - new filetyp unknown for debug files
 * - subject in cmd client
 * - lots of fixes for v2 protocoll handling
 *
 * Revision 1.21  2002/02/15 13:21:12  urut
 * first complete implementation of scep v2
 *
 * Revision 1.20  2002/02/15 10:12:20  urut
 * changed endEntityxxxx -> scepxxxx (because of v2 scep)
 *
 * Revision 1.19  2002/02/15 06:11:09  urut
 * merge with alani
 * lots of mods for incorporating full support for HTTP communication with SCEPD; Httpd client is now capable of reading multipart-mime (sort of....??) messages
 *
 * Revision 1.18  2002/02/13 12:55:22  alani
 * some changes
 *
 * Revision 1.17  2002/02/12 19:33:23  urut
 * replaced all println statements
 *
 * Revision 1.16  2002/02/12 14:01:17  urut
 * cmd client improved
 *
 * Revision 1.15  2002/02/07 17:08:50  urut
 * ability to send scep request over http added
 *
 * Revision 1.14  2002/02/07 13:16:34  urut
 * finished http CA cert get
 *
 * Revision 1.13  2002/02/07 12:40:55  urut
 * moved util-stuff to Utils.java
 * CA get implemented
 *
 * Revision 1.12  2002/01/31 14:07:32  urut
 * adjustments for new SignedData (that takes certificates as Sequences)
 *
 * Revision 1.11  2002/01/31 11:29:45  urut
 * new cmd option "-certificate"
 *
 * Revision 1.10  2002/01/24 00:11:31  urut
 * new flag -response added
 *
 * Revision 1.9  2002/01/17 11:51:30  urut
 * -key is now -privateKey, added new parameter -publicKey
 *
 * Revision 1.8  2002/01/10 20:22:01  urut
 * bug fixing
 *
 * Revision 1.7  2002/01/10 16:17:39  urut
 * fixed some bugs
 *
 * Revision 1.6  2002/01/10 15:09:38  alani
 * new attribute distinguishedName in ScepClient, scepReq now generates a transactionId
 *
 * Revision 1.2  2002/01/08 21:00:29  urut
 * added javadoc header with cvs tags to every file
 *
 * </pre>
 *
 * @since $Date: 2002/02/27 04:21:27 $
 * @author  
 * @version $Revision: 1.31 $
 */


public class CmdClient {

	public static void showHelp() {
                System.out.println("openscep java client - cmd line version");
                System.out.println("CmdClient [options] command");
                System.out.println("available commands: certRequest, certRep");
                System.out.println("------------------------------------------------------------");
                System.out.println("options should be provided in the following order:");
		System.out.println("-verbose ");
                System.out.println("-logLevel number      1, 2 or 3 (1=minimum, 3=maximum, default=3)");

		System.out.println("-requestType PKCS|SPKAC   only valid for version 2 (default is PKCS)");
                System.out.println("-version 1|2          SCEP Version (default 1)");
                
                
                System.out.println("-poll                 poll the server for a certificate");

                System.out.println("-subject string       subject string, eg. CN=xxxx  . only if type=SPKAC");
                System.out.println("-subjectProxy string       subject of the proxy, eg. CN=xxxx  . only if type=SPKAC");

                System.out.println("-community string     Community String (V2->proxy)");
                
                System.out.println("-caCertURL url        URL to get the CA cert eg http://..../ or file:///tmp/...");
                
		System.out.println("-privatekey filename  URL to get the private key");
                System.out.println("-pubkey filename      URL to get the public key (is automaticly extracted from the pkcs#10 request (if available)");
		System.out.println("-certReq filename     URL to get the certification request");
		
                //where to send/store request
                System.out.println("-serverURL theURL     URL of the server to which requests are sent eg http://..../");
                System.out.println("-request filename     File where the request is being stored before it is sent out");
                
                System.out.println("-response filename    manually supply a response from a SCEP server");

                System.out.println("-certificate filename Location (URL) where the CA signed end entity certificate is written to (in case of a SUCCESS response)");

	}

	public static void main(String[] args) {

		int i = 0, j;
		String arg;
		char flag;		

		String caCertFile = "";
                String caCertURL  = "";
                
                String keyPrivateFile    = "";
                String keyPubFile = "";
		String challenge  = "";
		String requestFile= "certReq.pem";
		String url        = "";
		String dname	  = "";
                String responseFile ="";
		String certificateFile = null;

                String request = "";

                String requestType = "PKCS";        //default PKCS
                
                int version = 1; //default is version 1
                
		boolean poll = false;
		boolean vflag = false;

                int logLevel=3;
                
		ScepClient sc = new ScepClient();

		while (i < args.length && args[i].startsWith("-")) {
			
			arg = args[i++];
			 
			if (arg.equals("-verbose")) {
				Log.info("verbose mode on");
				vflag = true;
				
			} else if (arg.equals("-logLevel")) {
				if (i < args.length) {
                                        try {
                                            logLevel = Integer.parseInt( args[i++] );
                                        } catch (Exception e) {
                                            Log.error("invalid log level,set to default 3");
                                            logLevel=3;
                                        }
				} else
					Log.error("-logLevel requires a number");
				if (vflag)
					Log.info("loglevel = " + logLevel);
                                Log.setLogLevel(logLevel);
			} else if (arg.equals("-version")) {
				if (i < args.length) {
                                        try {
                                            version = Integer.parseInt( args[i++] );
                                            if ( version == ScepClient.VERSION1 )
                                                sc.setScepVersion(ScepClient.VERSION1);
                                            else if ( version == ScepClient.VERSION2 )
                                                sc.setScepVersion(ScepClient.VERSION2);
                                            else 
                                                Log.error("invalid version,set to default");
                                        } catch (Exception e) {
                                            Log.error("invalid version,set to default 1");
                                            version=1;
                                        }
				} else
					Log.error("-version requires a number");
				if (vflag)
					Log.info("version = " + version);
                                //sc.setVersion(version);
			} else if (arg.equals("-requestType")) {
				if (i < args.length) {
                                        try {
                                            requestType = args[i++];
                                        } catch (Exception e) {
                                            Log.error("invalid request Type");
                                            requestType="PKCS";
                                        }
                                        
                                        if ( requestType.equals("PKCS") )
                                        {
                                            Log.debug("set request type to PKCS");
                                            sc.setRequestType( ScepClient.TYPE_PKCS10);
                                        }  else if ( requestType.equals("SPKAC") ) {
                                            Log.debug("set request type to SPKAC");
                                            sc.setRequestType( ScepClient.TYPE_SPKAC );
                                        } else {
                                            Log.error("invalid typ");
                                            return;
                                        }
				} else
					Log.error("-version requires a number");
				if (vflag)
					Log.info("version = " + version);
                                //sc.setVersion(version);
                        } else if (arg.equals("-community")) {
				if (i < args.length) {
                                        sc.setCommunityString(args[i++]);
				} else
					Log.error("-community requires a string");
                        } else if (arg.equals("-certificate")) {
				if (i < args.length) {
					certificateFile = new String ( args[i++] );
                                        try {
                                            sc.setCertificate( new URL(certificateFile) );
                                        } catch ( java.net.MalformedURLException e) {
                                            Log.error("invalid URL");
                                            e.printStackTrace();
                                        }
				} else
					Log.error("-certificate requires a filename");
				if (vflag)
					Log.info("certificate file = " + certificateFile );
			} else if (arg.equals("-response")) {
				if (i < args.length) {
					responseFile = args[i++];
					try {
						sc.loadResponseFile( responseFile );
					} catch ( java.io.FileNotFoundException e ) {
						Log.error("-response filename not found");
                                                return;
					} catch ( java.io.IOException e ) {
						Log.error("-response IOException occured");
                                                return;
					}
				} else
					Log.error("-response requires a filename");
				if (vflag)
					Log.info("response file = " + responseFile );

                        } else if (arg.equals("-caCertURL")) {
				if (i < args.length) {
					caCertURL = args[i++];
					try {
						sc.loadCA( caCertURL );
					} catch ( java.io.FileNotFoundException e ) {
						Log.error("-caCertURL invalid URL");
                                                //exit, can't continue
                                                return;
					} catch ( java.io.IOException e ) {
                                            //e.printStackTrace();
						Log.error("-caCertURL IOException occured (invalid URL) ");
                                                return;
					}
				} else
					Log.error("-caCertURL requires a url");
				if (vflag)
					Log.info("caCertURL = " + caCertURL );
					
			} else if (arg.equals("-pubkey")) {
				if (i < args.length) {
					keyPubFile = args[i++];
					try {
						sc.loadPublicKey( new URL(keyPubFile) );
					} catch ( java.io.FileNotFoundException e ) {
						Log.error("-pubkey filename not found");
                                                return;
					} catch ( java.io.IOException e ) {
						Log.error("-pubkey IOException occured");
                                                return;
					}
				} else
					Log.error("-key requires a filename");
				if (vflag)
					Log.info("pubkey file = " + keyPubFile );
			} else if (arg.equals("-privatekey")) {
				if (i < args.length) {
					keyPrivateFile = args[i++];
					try {
						sc.loadPrivateKey( new URL(keyPrivateFile) );
					} catch ( java.io.FileNotFoundException e ) {
						Log.error("-privatekey filename not found");
                                                return;
					} catch ( java.io.IOException e ) {
						Log.error("-privatekey IOException occured");
                                                return;
					}
				} else
					Log.error("-privatekey requires a filename");
				if (vflag)
					Log.info("private key file = " + keyPrivateFile );
					
			} else if (arg.equals("-certReq")) {
				if (i < args.length) {
					requestFile = args[i++];
					try {
						sc.loadRequest( new URL(requestFile) );
					} catch ( java.io.FileNotFoundException e ) {
						Log.error("-certReq filename not found");
                                                return;
					} catch ( java.io.IOException e ) {
						Log.error("-certReq IOException occured");
                                                return;
					}
				} else
					Log.error("-certReq requires a filename");
				if (vflag)
					Log.info("certReq file = " + requestFile );
					
			} else if (arg.equals("-serverURL")) {
				if (i < args.length) {
					url = args[i++];
					sc.setUrl( url );
				} else
					Log.error("-serverURL requires an url");
				if (vflag)
					Log.info("serverURL = " + url );

                        } else if (arg.equals("-request")) {
				if (i < args.length) {
					request = args[i++];
					sc.setRequest( request );
				} else
					Log.error("-request requires an filename");
				if (vflag)
					Log.info("requestL = " + request );
					
			} else if (arg.equals("-subject")) {
				if (i < args.length) {
                                    String x509String =  args[i++];
                                    sc.setSubject( new X509Name(x509String ));
                                        
                                    Log.debug("X509 subject structure is: " + ASN1Dump.dumpAsString( new X509Name( x509String)));
                                              
				} else
					Log.error("-subject requires a String");
		

			} else if (arg.equals("-subjectProxy")) {
				if (i < args.length) {
                                    String x509String =  args[i++];
                                    sc.setSubjectScep( new X509Name(x509String ));
                                        
                                    Log.debug("X509 subject scep structure is: " + ASN1Dump.dumpAsString( new X509Name( x509String)));
                                              
				} else
					Log.error("-subjectProxy requires a String");
			} 	


                        // use this type of check for a series of flag arguments
			else {
				for (j = 1; j < arg.length(); j++) {
					flag = arg.charAt(j);
					switch (flag) {
						case 'p' :
							if (vflag)
								Log.info("Option p");
							break;
						case 'v' :
							if (vflag)
								Log.info("Option v");
							break;
						default :
							Log.error("ParseCmdLine: illegal option " + flag);
							break;
					}
				}
			}
		}
		
		if (i < args.length) {	
	
			try {
				sc.Execute( args[i] );                                 
				
			} catch ( IOException e ) {
				e.printStackTrace();
			} catch ( ScepPending e ) {
                                Log.info("PENDING");
				//e.printStackTrace();
			} catch ( ScepError e ) {
				e.printStackTrace();
			}
                        
                        return;
		}
			
		showHelp();
		
	}

    private ScepClient lnkScepClient;
}
